# coding: utf-8
""" Some common things for tests.
"""
import unittest

from django.test import TestCase as BaseTestCase
from django.test.runner import DiscoverRunner as BaseTestRunner

from django_vest import config


class ThemebleTestSuite(unittest.TestSuite):
    """ Skip casess with countTestCases == 0.
    """
    def __init__(self, tests=()):
        tests = [t for t in tests if t.countTestCases()]
        super(ThemebleTestSuite, self).__init__(tests)

    def addTest(self, test):
        if test.countTestCases():
            super(ThemebleTestSuite, self).addTest(test)


class ThemebleTestLoader(unittest.TestLoader):
    suiteClass = ThemebleTestSuite

themebleTestLoader = ThemebleTestLoader()


class ThemebleTestRunner(BaseTestRunner):
    """ Our custom TestRunner.
    Set in your settings.py:

        ``TEST_RUNNER = 'django_vest.test.ThemebleTestRunner'``
    """
    test_suite = ThemebleTestSuite
    test_loader = themebleTestLoader

    def run_tests(self, test_labels, extra_tests=None, **kwargs):
        if self.verbosity > 1:
            print(u'>>> Run tests for theme: {}'.format(
                config.CURRENT_THEME))
        return super(ThemebleTestRunner, self).run_tests(
            test_labels, extra_tests, **kwargs)


class TestCase(BaseTestCase):
    """ Our new base TestCase.
    """
    # List of themes
    THEMES = None

    def countTestCases(self):
        themes = self.THEMES or []

        if not isinstance(themes, (list, tuple, set)):
            themes = [themes]

        if not themes or config.CURRENT_THEME in themes:
            return super(TestCase, self).countTestCases()

        return 0



# coding: utf-8
from django.conf import settings as django_settings


class Settings(object):
    """ Simple proxy above global settings of project.
    """
    def __getattr__(self, name):
        return getattr(django_settings, name)

    @property
    def CURRENT_THEME(self):
        return getattr(django_settings, 'CURRENT_THEME', None)

    @property
    def DEFAULT_THEME(self):
        return getattr(django_settings, 'DEFAULT_THEME', None)

settings = Settings()



# coding: utf-8
import inspect

from django_vest.config import settings

__ALL__ = ('themeble',)


def themeble(name, themes=None, global_context=None):
    """ Decorator for registering objects (i.e. functions, classes) for
    different themes.

    Params:

        * name - type of string. New global name for object
        * themes - for this themes ``obj`` will be have alias with given name
        * global_context - current decorator's global context


    Example:

    .. code:: python

        # my_app.forms.py

        @themeble(name='Form', themes=('dark_theme',))
        class DarkThemeForm(object):
            ''' Some kind of logic for dark_theme
            '''
            name = 'DarkThemeForm'


        @themeble(name='Form')
        class DefaultForm(object):
            ''' Default logic for all themes
            '''
            name = 'Default form'


    Now if settings.CURRENT_THEME == 'dark_theme':

    .. code:: python

        # my_app.views.py

        from my_app.forms import Form
        assert Form.name == 'DarkThemeForm'

    """
    def wrap(obj):
        context = global_context or inspect.stack()[1][0].f_globals

        if themes and settings.CURRENT_THEME in themes:
            context[name] = obj

        elif themes is None and name not in context:
            context[name] = obj

        return obj
    return wrap



# coding: utf-8
from django.utils._os import safe_join
from django.template.loaders.filesystem import Loader as BaseLoader

from django_vest.config import settings

__ALL__ = ('Loader',)


class ThemeLoaderMixin(object):
    """ Adding ability for work with themes.
    Use settings:
        - DEFAULT_THEME - default (main) theme
        - CURRENT_THEME

    Theme - this is folder inside TEMPLATE_DIRS.

    Supports inheritance only from DEFAULT_THEME like this:

    .. code:: html

        {% extends 'DEFAULT_THEME/app/index.html' %}
        ...


    """
    def get_template_sources(self, template_name, template_dirs=None):
        if not all([settings.DEFAULT_THEME, settings.CURRENT_THEME]):
            return super(ThemeLoaderMixin, self).get_template_sources(
                template_name, template_dirs)

        if not template_dirs:
            template_dirs = settings.TEMPLATE_DIRS

        _template_dirs = template_dirs

        template_name = template_name.replace('DEFAULT_THEME',
                                              settings.DEFAULT_THEME)

        # First we find templates in CURRENT_THEME folder
        template_dirs = [safe_join(t, settings.CURRENT_THEME)
                         for t in template_dirs]

        if settings.CURRENT_THEME != settings.DEFAULT_THEME:
            # If this is not default theme - add extra folders for search
            template_dirs.extend([safe_join(t, settings.DEFAULT_THEME)
                                  for t in _template_dirs])
            template_dirs.extend(_template_dirs)

        return super(ThemeLoaderMixin, self).get_template_sources(
            template_name, template_dirs)


class Loader(ThemeLoaderMixin, BaseLoader):
    """ Template loader class
    """



# coding: utf-8

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

requirements = [
    'Django>=1.5.1',
]

test_requirements = [
    'tox'
]

setup(
    name="django-vest",
    version="0.1.2a",

    author="Vladimir Savin",
    author_email="zero13cool@yandex.ru",
    url="https://github.com/zerc/django-vest",

    description="Extension for default template system for making inheritance more flexible. Adding some kind of themes.",
    long_description=readme,

    packages=[
        'django_vest',
    ],
    package_dir={'django_vest':
                 'django_vest'},
    install_requires=requirements,
    license="MIT",
    zip_safe=False,
    keywords='django-vest',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
